=== modified file 'cloudinit/options.go'
--- cloudinit/options.go	2013-11-01 04:00:55 +0000
+++ cloudinit/options.go	2013-11-15 08:47:05 +0000
@@ -31,6 +31,13 @@
 	cfg.set("apt_upgrade", yes, yes)
 }
 
+// AptUpgrade returns the value set by SetAptUpgrade, or
+// false if no call to SetAptUpgrade has been made.
+func (cfg *Config) AptUpgrade() bool {
+	update, _ := cfg.attrs["apt_upgrade"].(bool)
+	return update
+}
+
 // SetUpdate sets whether cloud-init runs "apt-get update"
 // on first boot.
 func (cfg *Config) SetAptUpdate(yes bool) {

=== added directory 'cloudinit/sshinit'
=== renamed file 'environs/manual/agent.go' => 'cloudinit/sshinit/configure.go'
--- environs/manual/agent.go	2013-11-14 08:27:15 +0000
+++ cloudinit/sshinit/configure.go	2013-11-15 08:47:05 +0000
@@ -1,7 +1,7 @@
 // Copyright 2013 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
-package manual
+package sshinit
 
 import (
 	"encoding/base64"
@@ -9,25 +9,29 @@
 	"os"
 	"strings"
 
-	corecloudinit "launchpad.net/juju-core/cloudinit"
-	"launchpad.net/juju-core/environs/cloudinit"
+	"launchpad.net/loggo"
+
+	"launchpad.net/juju-core/cloudinit"
 	"launchpad.net/juju-core/utils"
+	"launchpad.net/juju-core/utils/ssh"
 )
 
-// ProvisionMachineAgent connects to a machine over SSH,
-// and installs a machine agent.
-func ProvisionMachineAgent(host string, mcfg *cloudinit.MachineConfig) error {
+var logger = loggo.GetLogger("juju.cloudinit.sshinit")
+
+// Configure connects to the specified host over SSH,
+// and executes a script that carries out cloud-config.
+func Configure(host string, cfg *cloudinit.Config) error {
 	logger.Infof("Provisioning machine agent on %s", host)
-	script, err := provisionMachineAgentScript(mcfg)
+	script, err := generateScript(cfg)
 	if err != nil {
 		return err
 	}
 	scriptBase64 := base64.StdEncoding.EncodeToString([]byte(script))
 	script = fmt.Sprintf(`F=$(mktemp); echo %s | base64 -d > $F; . $F`, scriptBase64)
-	cmd := sshCommand(
+	cmd := ssh.Command(
 		host,
 		fmt.Sprintf("sudo bash -c '%s'", script),
-		allocateTTY,
+		ssh.AllocateTTY,
 	)
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
@@ -35,17 +39,9 @@
 	return cmd.Run()
 }
 
-// provisionMachineAgentScript generates the script necessary
-// to install a machine agent with the provided MachineConfig.
-func provisionMachineAgentScript(mcfg *cloudinit.MachineConfig) (string, error) {
-	// We generate a cloud-init config, which we'll then pull the runcmds
-	// and prerequisite packages out of. Rather than generating a cloud-config,
-	// we'll just generate a shell script.
-	cloudcfg := corecloudinit.New()
-	if err := cloudinit.Configure(mcfg, cloudcfg); err != nil {
-		return "", err
-	}
-
+// generateScript generates the script that applies
+// the specified cloud-config.
+func generateScript(cloudcfg *cloudinit.Config) (string, error) {
 	// TODO(axw): 2013-08-23 bug 1215777
 	// Carry out configuration for ssh-keys-per-user,
 	// machine-updates-authkeys, using cloud-init config.
@@ -87,7 +83,7 @@
 
 // addPackageCommands returns a slice of commands that, when run,
 // will add the required apt repositories and packages.
-func addPackageCommands(cfg *corecloudinit.Config) ([]string, error) {
+func addPackageCommands(cfg *cloudinit.Config) ([]string, error) {
 	var cmds []string
 	if len(cfg.AptSources()) > 0 {
 		// Ensure apt-add-repository is available.
@@ -104,11 +100,12 @@
 		}
 		cmds = append(cmds, "apt-add-repository -y "+utils.ShQuote(src.Source))
 	}
-	if len(cfg.AptSources()) > 0 {
+	if len(cfg.AptSources()) > 0 || cfg.AptUpdate() {
 		cmds = append(cmds, "apt-get -y update")
 	}
-	// Note: explicitly ignoring apt_upgrade, so as not to trample the target
-	// machine's existing configuration.
+	if cfg.AptUpgrade() {
+		cmds = append(cmds, "apt-get -y upgrade")
+	}
 	for _, pkg := range cfg.Packages() {
 		cmd := fmt.Sprintf("apt-get -y install %s", utils.ShQuote(pkg))
 		cmds = append(cmds, cmd)

=== renamed file 'environs/manual/agent_test.go' => 'cloudinit/sshinit/configure_test.go'
--- environs/manual/agent_test.go	2013-11-14 08:27:15 +0000
+++ cloudinit/sshinit/configure_test.go	2013-11-15 08:47:05 +0000
@@ -1,35 +1,46 @@
 // Copyright 2013 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
-package manual
+package sshinit
 
 import (
 	gc "launchpad.net/gocheck"
 
+	"launchpad.net/juju-core/cloudinit"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs"
-	"launchpad.net/juju-core/environs/cloudinit"
+	envcloudinit "launchpad.net/juju-core/environs/cloudinit"
 	"launchpad.net/juju-core/environs/config"
 	envtools "launchpad.net/juju-core/environs/tools"
-	_ "launchpad.net/juju-core/provider/dummy"
 	coretesting "launchpad.net/juju-core/testing"
 	"launchpad.net/juju-core/testing/testbase"
 	"launchpad.net/juju-core/tools"
 	"launchpad.net/juju-core/version"
 )
 
-type agentSuite struct {
+type configureSuite struct {
 	testbase.LoggingSuite
 }
 
-var _ = gc.Suite(&agentSuite{})
-
-func dummyConfig(c *gc.C, stateServer bool, vers version.Binary) *config.Config {
+var _ = gc.Suite(&configureSuite{})
+
+type testProvider struct {
+	environs.EnvironProvider
+}
+
+func (p *testProvider) SecretAttrs(cfg *config.Config) (map[string]string, error) {
+	return map[string]string{}, nil
+}
+
+func init() {
+	environs.RegisterProvider("sshinit", &testProvider{})
+}
+
+func testConfig(c *gc.C, stateServer bool, vers version.Binary) *config.Config {
 	testConfig, err := config.New(config.UseDefaults, coretesting.FakeConfig())
 	c.Assert(err, gc.IsNil)
 	testConfig, err = testConfig.Apply(map[string]interface{}{
-		"type":           "dummy",
-		"state-server":   stateServer,
+		"type":           "sshinit",
 		"default-series": vers.Series,
 		"agent-version":  vers.Number.String(),
 	})
@@ -37,8 +48,8 @@
 	return testConfig
 }
 
-func (s *agentSuite) getMachineConfig(c *gc.C, stateServer bool, vers version.Binary) *cloudinit.MachineConfig {
-	var mcfg *cloudinit.MachineConfig
+func (s *configureSuite) getCloudConfig(c *gc.C, stateServer bool, vers version.Binary) *cloudinit.Config {
+	var mcfg *envcloudinit.MachineConfig
 	if stateServer {
 		mcfg = environs.NewBootstrapMachineConfig("http://whatever/dotcom")
 	} else {
@@ -48,10 +59,13 @@
 		Version: vers,
 		URL:     "file:///var/lib/juju/storage/" + envtools.StorageName(vers),
 	}
-	environConfig := dummyConfig(c, stateServer, vers)
+	environConfig := testConfig(c, stateServer, vers)
 	err := environs.FinishMachineConfig(mcfg, environConfig, constraints.Value{})
 	c.Assert(err, gc.IsNil)
-	return mcfg
+	cloudcfg := cloudinit.New()
+	err = envcloudinit.Configure(mcfg, cloudcfg)
+	c.Assert(err, gc.IsNil)
+	return cloudcfg
 }
 
 var allSeries = [...]string{"precise", "quantal", "raring", "saucy"}
@@ -63,10 +77,10 @@
 	return gc.Not(checker)
 }
 
-func (s *agentSuite) TestAptSources(c *gc.C) {
+func (s *configureSuite) TestAptSources(c *gc.C) {
 	for _, series := range allSeries {
 		vers := version.MustParseBinary("1.16.0-" + series + "-amd64")
-		script, err := provisionMachineAgentScript(s.getMachineConfig(c, true, vers))
+		script, err := generateScript(s.getCloudConfig(c, true, vers))
 		c.Assert(err, gc.IsNil)
 
 		// Only Precise requires the cloud-tools pocket.

=== added file 'cloudinit/sshinit/suite_test.go'
--- cloudinit/sshinit/suite_test.go	1970-01-01 00:00:00 +0000
+++ cloudinit/sshinit/suite_test.go	2013-11-15 08:47:05 +0000
@@ -0,0 +1,14 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
+package sshinit
+
+import (
+	stdtesting "testing"
+
+	gc "launchpad.net/gocheck"
+)
+
+func Test(t *stdtesting.T) {
+	gc.TestingT(t)
+}

=== modified file 'environs/cloudinit.go'
--- environs/cloudinit.go	2013-11-14 08:27:15 +0000
+++ environs/cloudinit.go	2013-11-15 08:47:05 +0000
@@ -130,9 +130,16 @@
 	for _, script := range additionalScripts {
 		cloudcfg.AddRunCmd(script)
 	}
-	err := cloudinit.Configure(cfg, cloudcfg)
-	if err != nil {
-		return nil, err
+	// When bootstrapping, we only want to apt-get update/upgrade
+	// and setup the SSH keys. The rest we leave to environs/manual.
+	if cfg.StateServer {
+		if err := cloudinit.ConfigureBase(cfg.AuthorizedKeys, cloudcfg); err != nil {
+			return nil, err
+		}
+	} else {
+		if err := cloudinit.Configure(cfg, cloudcfg); err != nil {
+			return nil, err
+		}
 	}
 	data, err := cloudcfg.Render()
 	logger.Tracef("Generated cloud init:\n%s", string(data))

=== modified file 'environs/cloudinit/cloudinit.go'
--- environs/cloudinit/cloudinit.go	2013-11-14 08:27:15 +0000
+++ environs/cloudinit/cloudinit.go	2013-11-15 08:47:05 +0000
@@ -138,14 +138,13 @@
 // to initialise a base OS image, such that it can be connected to
 // via SSH.
 //
-// Note: we do an apt update/upgrade here to bring the base OS image
-// up to date; we do not add any sources or packages until ConfigureJuju.
-// The reason for this is that apt failures are a common source of
-// bootstrap failure, so we want that to be visible.
+// Note: we don't do apt update/upgrade here so as not to have to wait on
+// apt to finish when performing the second half of image initialisation.
+// Doing it later brings the benefit of feedback in the face of errors,
+// but adds to the running time of initialisation due to lack of activity
+// between image bringup and start of agent installation.
 func ConfigureBase(authorizedKeys string, c *cloudinit.Config) error {
 	c.AddSSHAuthorizedKeys(authorizedKeys)
-	c.SetAptUpgrade(true)
-	c.SetAptUpdate(true)
 	c.SetOutput(cloudinit.OutAll, "| tee -a /var/log/cloud-init-output.log", "")
 	return nil
 }
@@ -156,7 +155,14 @@
 	if err := verifyConfig(cfg); err != nil {
 		return err
 	}
+
+	// Bring packages up-to-date.
+	c.SetAptUpdate(true)
+	c.SetAptUpgrade(true)
+
+	// juju requires git for managing charm directories.
 	c.AddPackage("git")
+
 	// Perfectly reasonable to install lxc on environment instances and kvm
 	// containers.
 	if cfg.MachineContainerType != instance.LXC {

=== modified file 'environs/manual/bootstrap.go'
--- environs/manual/bootstrap.go	2013-11-14 08:27:15 +0000
+++ environs/manual/bootstrap.go	2013-11-15 08:47:05 +0000
@@ -120,5 +120,5 @@
 	for k, v := range agentEnv {
 		mcfg.AgentEnvironment[k] = v
 	}
-	return ProvisionMachineAgent(args.Host, mcfg)
+	return provisionMachineAgent(args.Host, mcfg)
 }

=== modified file 'environs/manual/detection.go'
--- environs/manual/detection.go	2013-09-16 03:24:54 +0000
+++ environs/manual/detection.go	2013-11-15 08:47:05 +0000
@@ -12,6 +12,7 @@
 
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/utils"
+	"launchpad.net/juju-core/utils/ssh"
 )
 
 // checkProvisionedScript is the script to run on the remote machine
@@ -25,7 +26,7 @@
 // exist on the host machine.
 func checkProvisioned(sshHost string) (bool, error) {
 	logger.Infof("Checking if %s is already provisioned", sshHost)
-	cmd := sshCommand(sshHost, fmt.Sprintf("bash -c %s", utils.ShQuote(checkProvisionedScript)))
+	cmd := ssh.Command(sshHost, fmt.Sprintf("bash -c %s", utils.ShQuote(checkProvisionedScript)))
 	var stdout, stderr bytes.Buffer
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
@@ -50,7 +51,7 @@
 // connecting to the machine and executing a bash script.
 func detectSeriesAndHardwareCharacteristics(sshHost string) (hc instance.HardwareCharacteristics, series string, err error) {
 	logger.Infof("Detecting series and characteristics on %s", sshHost)
-	cmd := sshCommand(sshHost, "bash")
+	cmd := ssh.Command(sshHost, "bash")
 	cmd.Stdin = bytes.NewBufferString(detectionScript)
 	out, err := cmd.CombinedOutput()
 	if err != nil {

=== modified file 'environs/manual/provisioner.go'
--- environs/manual/provisioner.go	2013-11-14 08:27:15 +0000
+++ environs/manual/provisioner.go	2013-11-15 08:47:05 +0000
@@ -11,6 +11,8 @@
 
 	"launchpad.net/loggo"
 
+	coreCloudinit "launchpad.net/juju-core/cloudinit"
+	"launchpad.net/juju-core/cloudinit/sshinit"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/environs/cloudinit"
@@ -89,7 +91,7 @@
 	}
 
 	// Finally, provision the machine agent.
-	err = ProvisionMachineAgent(args.Host, mcfg)
+	err = provisionMachineAgent(args.Host, mcfg)
 	if err != nil {
 		return machineId, err
 	}
@@ -202,3 +204,14 @@
 	}
 	return mcfg, nil
 }
+
+func provisionMachineAgent(host string, mcfg *cloudinit.MachineConfig) error {
+	cloudcfg := coreCloudinit.New()
+	if err := cloudinit.ConfigureJuju(mcfg, cloudcfg); err != nil {
+		return err
+	}
+	// Explicitly disabling apt_upgrade so as not to trample
+	// the target machine's existing configuration.
+	cloudcfg.SetAptUpgrade(false)
+	return sshinit.Configure(host, cloudcfg)
+}

=== modified file 'provider/common/bootstrap.go'
--- provider/common/bootstrap.go	2013-10-02 19:30:07 +0000
+++ provider/common/bootstrap.go	2013-11-15 08:47:05 +0000
@@ -5,11 +5,16 @@
 
 import (
 	"fmt"
+	"net"
+	"time"
 
 	"launchpad.net/loggo"
 
+	coreCloudinit "launchpad.net/juju-core/cloudinit"
+	"launchpad.net/juju-core/cloudinit/sshinit"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs"
+	"launchpad.net/juju-core/environs/cloudinit"
 	"launchpad.net/juju-core/instance"
 	coretools "launchpad.net/juju-core/tools"
 )
@@ -19,11 +24,22 @@
 // Bootstrap is a common implementation of the Bootstrap method defined on
 // environs.Environ; we strongly recommend that this implementation be used
 // when writing a new provider.
-func Bootstrap(env environs.Environ, cons constraints.Value, possibleTools coretools.List) error {
+func Bootstrap(env environs.Environ, cons constraints.Value, possibleTools coretools.List) (err error) {
 	// TODO make safe in the case of racing Bootstraps
 	// If two Bootstraps are called concurrently, there's
 	// no way to make sure that only one succeeds.
 
+	var inst instance.Instance
+	defer func() {
+		if err == nil || inst == nil {
+			return
+		}
+		if stoperr := env.StopInstances([]instance.Instance{inst}); stoperr != nil {
+			// Failure upon failure.  Log it, but return the original error.
+			logger.Errorf("cannot stop failed bootstrap instance %q: %v", inst.Id(), stoperr)
+		}
+	}()
+
 	// Create an empty bootstrap state file so we can get its URL.
 	// It will be updated with the instance id and hardware characteristics
 	// after the bootstrap instance is started.
@@ -32,7 +48,8 @@
 		return err
 	}
 	machineConfig := environs.NewBootstrapMachineConfig(stateFileURL)
-	inst, hw, err := env.StartInstance(cons, possibleTools, machineConfig)
+	var hw *instance.HardwareCharacteristics
+	inst, hw, err = env.StartInstance(cons, possibleTools, machineConfig)
 	if err != nil {
 		return fmt.Errorf("cannot start bootstrap instance: %v", err)
 	}
@@ -47,12 +64,31 @@
 			Characteristics: characteristics,
 		})
 	if err != nil {
-		stoperr := env.StopInstances([]instance.Instance{inst})
-		if stoperr != nil {
-			// Failure upon failure.  Log it, but return the original error.
-			logger.Errorf("cannot stop failed bootstrap instance %q: %v", inst.Id(), stoperr)
-		}
 		return fmt.Errorf("cannot save state: %v", err)
 	}
-	return nil
+	logger.Infof("waiting for DNS name for instance %q", inst.Id())
+	dnsName, err := WaitDNSName(inst)
+	if err != nil {
+		return err
+	}
+	// Wait until we can open a connection to port 22.
+	connected := false
+	for a := LongAttempt.Start(); !connected && a.Next(); {
+		logger.Infof("attempting to connect to %s:22...", dnsName)
+		conn, err := net.DialTimeout("tcp", dnsName+":22", 5*time.Second)
+		if err == nil {
+			conn.Close()
+			connected = true
+		} else {
+			logger.Errorf("failed to connect: %v", err)
+		}
+	}
+	if !connected {
+		return fmt.Errorf("could not connect to host")
+	}
+	cloudcfg := coreCloudinit.New()
+	if err := cloudinit.ConfigureJuju(machineConfig, cloudcfg); err != nil {
+		return err
+	}
+	return sshinit.Configure("ubuntu@"+dnsName, cloudcfg)
 }

=== added directory 'utils/ssh'
=== renamed file 'environs/manual/ssh.go' => 'utils/ssh/ssh.go'
--- environs/manual/ssh.go	2013-09-12 02:04:05 +0000
+++ utils/ssh/ssh.go	2013-11-15 08:47:05 +0000
@@ -1,22 +1,20 @@
 // Copyright 2013 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
-package manual
+package ssh
 
 import (
 	"os/exec"
 )
 
-type sshOption []string
-
-var allocateTTY sshOption = []string{"-t"}
-
-// TODO(axw) 2013-09-12 bug #1224230
-// Move this to a common package for use in cmd/juju, and others.
-var commonSSHOptions = []string{"-o", "StrictHostKeyChecking no"}
-
-func sshCommand(host string, command string, options ...sshOption) *exec.Cmd {
-	args := append([]string{}, commonSSHOptions...)
+type Option []string
+
+var AllocateTTY Option = []string{"-t"}
+
+var commonOptions = []string{"-o", "StrictHostKeyChecking no"}
+
+func Command(host string, command string, options ...Option) *exec.Cmd {
+	args := append([]string{}, commonOptions...)
 	for _, option := range options {
 		args = append(args, option...)
 	}

